from argparse import ArgumentParser, RawTextHelpFormatter
import importlib
from spl_widgets import tuner, stk_swx, widgets_help, update_widgets, gorilla_clean

parser_desc = "Run one of the python modules created by CTSF for the Barnard Speech Perception Laboratory"
arg_help = "(Positional) The module to run. Options: \
    \n\t- tuner \n\t- stk_swx \n\t- gorilla_clean \n\t- widgets_help \n\t- update_widgets"

mods={
    "tuner": tuner,
    "stk_swx": stk_swx,
    "widgets_help": widgets_help,
    "update_widgets": update_widgets,
    "gorilla_clean": gorilla_clean
}

parser = ArgumentParser(
    "runmod",
    description=parser_desc,
    formatter_class=RawTextHelpFormatter
)
parser.add_argument("mod", metavar="M", help=arg_help)

args = parser.parse_args()
cmd = args.mod

# Programmatically import and run the desired module, instead of importing everything
try:
    mods[cmd].main()
except ModuleNotFoundError:
    print("Bad command: " + repr(cmd))


import pandas as pd
import numpy as np
from io import StringIO
from subprocess import run
from datetime import datetime
from spl_widgets.misc_util import *

def tune_cols(filepath: str, interval: int, scale, tune_freqs):

    df = pd.read_csv(StringIO(open(filepath,'r').read()), sep='\t')

    formants = len(df.columns)//2
    size = len(df.index)
    out_df = pd.DataFrame(df.iloc[:,0])

    scale_notes = construct_note_freqs(scale)

    for i in range(formants):

        amp_col = df.iloc[:,2*i+2]
        freq_col = [n * (amp_col[j]>0) for j, n in enumerate(df.iloc[:,2*i+1])]
        new_col =[]
 
        for slice_start in range(0, size, interval):

            slice_end = slice_start + interval
            freq_slice = freq_col[slice_start:slice_end]

            nz_amps = np.count_nonzero(amp_col[slice_start:slice_end])
            range_freq = sum(freq_slice) / max(1, nz_amps)

            if tune_freqs is True and range_freq != 0:
                range_freq = get_closest(scale_notes, range_freq)

            new_col+=[range_freq]*len(freq_slice)

        out_df[f'F{i}']=new_col; out_df[f'A{i}']=amp_col

    out_df.columns = [formants]+['']*(2*formants)

    # Makes, populates and creates a folder for tuned file
    now_str = f'{datetime.now():%Y-%m-%d_%H.%M.%S.%f}'
    filename = filepath[filepath.rfind('/'):-4]

    out_dir_filepath = filepath[:filepath.rfind('/')]+f'/tuning_done_{now_str}'
    run(['mkdir', out_dir_filepath], capture_output=True)

    out_df.to_csv(f'{out_dir_filepath}/{filename}_tuned.tsv', index=False, sep='\t')
    run(['mv', f'{out_dir_filepath}/{filename}_tuned.tsv', f'{out_dir_filepath}/{filename}_tuned.swx'], capture_output=True)

    # Creates params.txt file
    notes_tuning = hex(sum([2**i*(i+1 in scale) for i in range(12)]))[2:]
    tuning_key = f"{int(tune_freqs)}{interval}-{notes_tuning}"

    args=[
        'Tuning Parameters:',
        '*'*20,
        f'Interval: {10*interval}ms (setting: {interval})',
        f'Scale: {num_scale_to_strs(scale) if tune_freqs else "NOT TUNED"}',
        f"Tune Frequencies: {tune_freqs}",
        f'Tuning Key: {tuning_key}'
        ]
    
    with open(f'{out_dir_filepath}/params.txt','w') as writer: writer.write('\n'.join(args))\
        
    return out_dir_filepath


widget_desc={
    "stk_swx":"Usage: stk_swx [folder]\n\nConverts a desired file of type .stk to a MATLAB-ready .swx file.\n\nIf the optional flag 'folder' is used does the same to each .stk file in a user-specified directory.",
    "gorilla_clean":"Usage: gorilla_clean[folder]\n\nParses a selected folder of subject data returned by gorilla's survey tool into a cleaner .xlsx file with readable results.\n\nIf the optional flag 'folder' is used does the same to every subject data folder in a user-specified directory.",
    "tuner":"Usage: tuner \n\nTakes no flags. GUI program to allow for musical frequency forcing of SWX files with documentation of the parameters used.",
    "update_widgets":"Usage: update_widgets\n\nTakes no flags. Brings the package up-to-date, ensuring all widgets are downloaded in their most recent-versions and properly aliased.",
    "widgets_help":"Usage: widgets_help\n\nTakes no flags. Calls this function, printing messages detailing the use and syntax of each widget/function in this package."
}

menustr = "HELP MENU: CTSF's PYTHON WIDGETS"

def main():
    print(f"{menustr}\n{'*'*len(menustr)}\n")
    for k,v in widget_desc.items():
        print(f"{k}:\n{'*'*(len(k)+1)}\n{v}\n")


import pandas as pd
import tkinter as tk
from tkinter import filedialog
import sys
from subprocess import run

data_modules={
    "awk4":"Cooldown",
    "l5it":"Sentence Transcription",
    "9bbq": "Headphone Test"
}

desired_c_CHECK = ["Response", "ANSWER", "Correct","audio"]
desired_c_DEFAULT = ["Response", "audio"]

class Subject():

    id: int
    source: str
    outfp: str
    tasks: "dict[str,pd.DataFrame]" = {}

    def __init__(self, **kwargs):
        self.source = kwargs.get("source") or self.get_source_by_filedialog()

        self.outfp = kwargs.get("out") or self.source

        self.id = int(self.source[self.source.rfind("_")+1:])

    def get_source_by_filedialog(self):
        return filedialog.askdirectory(title="Subject Data Folder: ")

    def parse(self):
        if self.source == '': return False
        for task_key in data_modules.keys():
            filepath_to_task = f"{self.source}/{self.source[self.source.rfind('/'):self.source.rfind('_')]}_task-{task_key}-{self.id}.xlsx"
            self.tasks[task_key] = pd.read_excel(filepath_to_task)

            self.parse_file(task_key)
        
        self.write_to_excel()

    def parse_file(self, task_key):
        df = self.tasks[task_key]
        desired_c = [desired_c_DEFAULT,desired_c_CHECK][task_key == "9bbq"]

        df_formatted = df[desired_c].loc[[type(n) == str and "response" in n for n in df["Zone Type"]]]

        if task_key == "9bbq":
            pct_correct = str(round(sum(df_formatted["Correct"])/len(df_formatted.index)*100,2))+"%"
            pct_correct_row = pd.DataFrame([["PERCENT CORRECT","","",pct_correct]], columns=desired_c)
            df_formatted = pd.concat([df_formatted, pct_correct_row],axis=0)

        self.tasks[task_key] = df_formatted

    def write_to_excel(self):
        writer = pd.ExcelWriter(f"{self.outfp}/{self.id}_CLEANED.xlsx", engine="xlsxwriter")

        for task_key, df in self.tasks.items():
            df.to_excel(writer, sheet_name = data_modules[task_key], index=False)

        writer.save()


def main():
    
    root = tk.Tk()
    root.focus_force()
    root.wm_geometry("0x0+0+0")

    args = sys.argv
    if 'folder' in args:
        data_dir = filedialog.askdirectory(title="Directory containing subject folders: ")
        if data_dir == '': return False

        in_dir = str(run(["ls",data_dir],capture_output=True).stdout)[2:-2].split('\\n')
        
        for item in in_dir:
            if item.startswith("data_exp"):
                subj = Subject(source=f"{data_dir}/{item}", out=data_dir)
                subj.parse()

    else:
        subj1 = Subject()
        subj1.parse()

if __name__ == "__main__":
    main()


import pandas as pd
from io import StringIO
import tkinter as tk
from tkinter import filedialog
import sys
from subprocess import run

class MalformedFileError(Exception): pass

def parse_df(df: pd.DataFrame) -> "tuple[int, pd.DataFrame]":   # Returns the desired columns (active formant freq and amp) from an inputted pd.DataFrame

	df["f0"] = [n*10 for n in range(len(df.index))]
	for fmt in range(4,7):                              		# adds only the formants that are active (changing)

		amp_col = df[f"a{fmt}f"]
		for amp_val in amp_col:
			if amp_val != 30:                           # Check for non-baseline amplitude indicating active formant
				break

		else:
			cols = ["f0"]
			for x in range(1,fmt): cols.extend([f"f{x}",f"a{x}f"])
			return fmt-1, df[cols]

def stk_to_swx(filepath):
	multipliers = [.7,.4,.2,.1,0.5]

	# --- Read .stk file, and take the info to a dataframe --- #
	with open(filepath, 'r') as reader:
		file_content = ''.join(reader.readlines()[8:])
		
	try:
		(formants, df) = parse_df( pd.read_csv(StringIO(file_content), sep='\t') )
	except Exception:
		raise MalformedFileError("File is malformed and does not conform to generic .stk format")

	amp_cols = [f"a{f+1}f" for f in range(formants)]			# get amp col headers and number of formants

	# --- Correct amplitude vals --- #
	amp_max = [max(df[col]) for col in amp_cols]
	for i in range(formants):
		def scalemult(cell):
			if cell<=30: return 0
			return round(multipliers[i] * (amp_max[i] / cell), 4)
	
		colnm = amp_cols[i]
		col = df[colnm]
		df[colnm] = list(map(scalemult, col))

	# --- Prettifying and output --- #
	df.columns = [formants]+['']*(formants*2)
	out_fp = filepath[:-4]

	df.to_csv(f"{out_fp}.tsv", index=False, sep='\t')
	run(["mv", f"{out_fp}.tsv", f"{out_fp}.swx"], capture_output=False)

	return out_fp


def main():
	# --- Get filepath to .stk file --- #
	root = tk.Tk()
	root.focus_force()
	root.wm_geometry("0x0+0+0")

	if "folder" in sys.argv:
		out_fp = filepath = filedialog.askdirectory()
		if filepath == '': return False

		files_in_dir = str(run(["ls", filepath], capture_output=True).stdout)[2:-1].split('\\n')[:-1]		# gets and parses terminal ls output into list of files in directory
		operable_files = [file for file in files_in_dir if file.endswith(".stk")]							# gets files with .stk extension

		for file in operable_files:
			stk_to_swx(f'{filepath}/{file}')
			print(f"File {file} has been converted")

		print("\nCompatible files have been successfully converted")

	else:
		filepath=filedialog.askopenfilename(filetypes=[('',".stk")])
		if filepath == '': return False

		out_fp = stk_to_swx(filepath)
		print('\n'+f"File written to {out_fp}.swx.")

	# --- Display completion message, open .swx file/dir if user chooses --- #
	if input("Open Output? (y/n): ").lower() == 'y':
		run(["open", f"{out_fp}.swx"], capture_output=False)


if __name__ == "__main__":
	main()


import os

def main():
    os.system("pip3 -q install spl_widgets --upgrade")
    os.system("pip -q install spl_widgets --upgrade")

    zprofile = f"{os.getenv('HOME')}/.zprofile"
    with open(zprofile,"r") as reader:
        lines = reader.readlines()

    newlines = list(filter(lambda ln: ("spl_widgets" not in ln), lines))
    newlines.append('alias runmod="python3 -m spl_widgets"\n')

    with open(zprofile, "w") as writer:
        writer.writelines(newlines)

if __name__ == "__main__": main()


from tkinter import *
from tkinter import filedialog, ttk
from spl_widgets.misc_util import *
from spl_widgets.tune_freq import tune_cols
from subprocess import run

class RadioFrame(Frame):

    def __init__(self, master: Frame, options: "list[str]", label: str, onchange: "function", orientation="v") -> Frame:
        Frame.__init__(self, master)

        self.parent = master
        self.variable = IntVar()
        self.onchange = lambda: onchange(self.variable) if onchange else None

        packside = "top" if orientation == "v" else "left"
        anchorside = W if orientation == "v" else N

        self.label = Label(self,text=label)
        self.label.pack(side="top", anchor=W)

        self.radios = [Radiobutton(self,text=o, variable=self.variable, value=i, command=self.onchange) for i,o in enumerate(options)]
        for b in self.radios: b.pack(side=packside, anchor=anchorside)

    def disable(self):
        for b in self.radios: b.configure(state="disabled")

    def enable(self):
        for b in self.radios: b.configure(state="normal")

def getscale(Key: str):

    tune_freqs = int(Key[0])
    interval = int(Key[1:3])

    scale_bin = bin(int(Key[-3:],base=16))[2:]
    scale_bin = '0'*(12-len(scale_bin))+scale_bin
    scale_list = [i for i in range(1,13) if scale_bin[-i] == "1"]

    return tune_freqs, interval, scale_list

def main():

    root=Tk()
    root.title("CTSF's SWX Tuner")
    root.geometry("470x330+50+50")
    root.resizable(False, False)

    intervalVar = StringVar()
    fileVar = StringVar()
    scaleVar = StringVar(value="C Major Scale")
    keyVar = StringVar()

    tuneTypeVar = IntVar()
    showScalesVar = IntVar()
    noteVars = [IntVar() for _ in range(len(notes))]
    tuneFreqsVar = IntVar(value=1)

    cb_values = [f"{n} Major Scale" for n in notes]
    cb_values_ext = [f"{notes[i]} {n}" for n in default_scales.keys() for i in range(len(notes))]

    def validateKey(*args):
        key = keyVar.get(); newkey = key
        invalidKey = (
            len(key) > 7 or
            (4<len(key)<7 and not (key[-1].isdigit() or key[-1].lower() in 'abcdef')) or
            (len(key) == 4 and key[-1] != "-") or
            1<len(key)<4 and not key[-1].isdigit() or
            (len(key)==1 and key[0] not in '01'))

        if invalidKey: newkey = key[:-1]
        keyVar.set(newkey.upper())

    def processTuningKey():

        if len(keyVar.get())!=7: return -1

        tune_freqs, interval, scale_list = getscale(keyVar.get())
        tuneFreqsVar.set(tune_freqs)
        intervalVar.set(interval)
        selectorRadiosFrame.radios[1].invoke()

        for i,n in enumerate(noteVars):
            n.set(i+1 in scale_list)

    def limitIntervalLen(*args):
        interval_in = intervalVar.get()
        if len(interval_in)>2 or (len(interval_in) >0 and not interval_in[-1].isdigit()):
            intervalVar.set(interval_in[:-1])

    intervalVar.trace('w',limitIntervalLen)

    def disableScaleTuning():
        scaleCombobox.configure(state="disabled")
        advScaleCheckbox.configure(state="disabled")

    def disableNoteTuning():
        for noteCheckbox in noteButtons:
            noteCheckbox.configure(state="disabled")

    def enableScaleTuning():
        scaleCombobox.configure(state="readonly")
        advScaleCheckbox.configure(state="normal")

    def enableNoteTuning():
        for noteCheckbox in noteButtons:
            noteCheckbox.configure(state="normal")

    def showsel(v: IntVar):

        if v.get():
            disableScaleTuning()
            enableNoteTuning()
        else:
            disableNoteTuning()
            enableScaleTuning()

    def getFile():
        if fileTypeVar.get():
            file = filedialog.askdirectory(
                title="Select Parent Directory of Files to Tune: "
            )
            if file!="": fileName.configure(text=f"Directory: {file[file.rfind('/')+1:]}")
        else:
            file = filedialog.askopenfilename(
                title="Select File to Tune",
                filetypes=[("SWX Files","*.swx")]
                )
            if file != "": fileName.configure(text=f'File: {file[file.rfind("/")+1:]}')

        fileVar.set(file)

    def changeFileType(v: IntVar):
        if v.get():
            fileLabel.configure(text="Set Path to Tune: ")
            fileVar.set(""); fileName.configure(text="")
        else:
            fileLabel.configure(text="Set File to Tune: ")
            fileVar.set(""); fileName.configure(text="")

    def changeScalesShown():
        showAdditional = showScalesVar.get()
        if showAdditional: scaleCombobox.config(values=cb_values_ext)
        else:
            scaleCombobox.config(values=cb_values)
            if scaleVar.get() not in cb_values: scaleVar.set("C Major Scale")

    def changeTuneFreqs():
        tuneFreqs = tuneFreqsVar.get()
        if tuneFreqs: 
            selectorRadiosFrame.enable()
            if tuneTypeVar.get(): enableNoteTuning()
            else: enableScaleTuning()
        else:
            selectorRadiosFrame.disable()
            disableNoteTuning(); disableScaleTuning()

    def tuneWithData():
        filepath = fileVar.get()
        interval = int(intervalVar.get())
        tune_freqs = tuneFreqsVar.get()

        tuneType = tuneTypeVar.get()        # 0: scale, 1: notes
        if tuneType == 0:
            scale_note, scale_type = scaleVar.get().split(' ',maxsplit=1)
            scale = construct_default_scale(notes.index(scale_note)+1, scale_type)
        else:
            scale = [i+1 for i in range(len(noteButtons)) if noteVars[i].get()]

        can_proceed = (filepath and interval and (scale or not tune_freqs))
        if can_proceed:
            if fileTypeVar.get() == 0:   # 1: dir, 0: file
                outfilepath = tune_cols(filepath, interval, scale, bool(tune_freqs))
                run(['open', outfilepath], capture_output=True)
            else:
                filesInDir = [n for n in str(run(['ls',filepath],capture_output=True).stdout)[2:].split('\\n') if n.endswith('.swx')]
                for f in filesInDir: tune_cols(f"{filepath}/{f}", interval, scale, bool(tune_freqs))
                run(['open', filepath], capture_output=True)

    optionsFrame = Frame(root, borderwidth=10); optionsFrame.pack(side='left', anchor=N)
    tuningFrame = Frame(root, pady=5); tuningFrame.pack(side='right', anchor=N)

    # File Input
    fileFrame = Frame(optionsFrame, borderwidth=5)
    fileFrame.pack(side='top', anchor=W)

    fileRadioFrame = RadioFrame(fileFrame, options=['Single file', 'Directory'], label="Object to Tune:", orientation="h", onchange=changeFileType)
    fileRadioFrame.configure(borderwidth=3); fileRadioFrame.pack(side='top', anchor=W); fileTypeVar = fileRadioFrame.variable

    fileInputFrame = Frame(fileFrame); fileInputFrame.pack(side='top')

    fileLabel = Label(fileInputFrame, text="Set File to Tune:")
    fileLabel.pack(side='left', anchor=W)

    fileInput = Button(fileInputFrame, text="Browse...", command=getFile)
    fileInput.pack(side='left', anchor=W)

    fileName = Label(fileFrame)
    fileName.pack(side='top', anchor=W)


    # Interval Input
    intervalFrame = Frame(optionsFrame, borderwidth=3)
    intervalFrame.pack(side='top', anchor=W)

    intervalInputLabel = Label(intervalFrame, text="Tuning interval (x10ms): ")
    intervalInputLabel.pack(side='left',anchor=W)

    intervalInput = Entry(intervalFrame, width=2, textvariable=intervalVar)
    intervalInput.pack(side='right',anchor=S)


    # Submit Button
    submitButton = Button(optionsFrame, text="Tune File", command=tuneWithData, borderwidth=5)
    submitButton.pack(side='bottom',anchor=W)

    # Selector Radios
    selectorFrame = Frame(optionsFrame, borderwidth=3)
    selectorFrame.pack(side='top', anchor=W)

    tuneFreqsCheckbox = Checkbutton(selectorFrame, text="Tune Frequencies", variable=tuneFreqsVar, onvalue=1, offvalue=0, command=changeTuneFreqs)
    tuneFreqsCheckbox.pack(anchor=W, side='top')

    selectorRadiosFrame = RadioFrame(selectorFrame, options=["Scale", "Notes"], label="Tune File By:", onchange=showsel)
    tuneTypeVar = selectorRadiosFrame.variable; selectorRadiosFrame.pack(side="top", anchor=W)

    # Scale radio combobox
    scaleRadioFrame = Frame(tuningFrame, borderwidth=10)
    scaleRadioFrame.pack(anchor=N)

    scaleRadioLabel = Label(scaleRadioFrame, text="Select scale to tune to: ")
    scaleRadioLabel.pack(anchor=W)

    scaleCombobox = ttk.Combobox(scaleRadioFrame, textvariable = scaleVar, values=cb_values, state = "readonly")
    scaleCombobox.pack(anchor=W)

    advScaleCheckbox = Checkbutton(scaleRadioFrame, command=changeScalesShown, text="Show additional scales",onvalue=1,offvalue=0, variable=showScalesVar)
    advScaleCheckbox.pack(anchor=W)

    # Note checkbuttons
    noteButtonsFrame = Frame(tuningFrame, padx=10); noteButtonsFrame.pack(anchor=SW)
    bottomNotesFrame=Frame(noteButtonsFrame); bottomNotesFrame.pack(side='bottom')

    leftSideFrame = Frame(bottomNotesFrame, borderwidth=3, padx=10); leftSideFrame.pack(side='left', anchor=W)
    rightSideFrame = Frame(bottomNotesFrame, borderwidth=3,padx=10); rightSideFrame.pack(side='right', anchor=W)

    noteButtonsLabel = Label(noteButtonsFrame, text="Select note(s) to tune to: ")
    noteButtonsLabel.pack(anchor=N)

    noteButtons = [Checkbutton([leftSideFrame,rightSideFrame][i>5], text=n, variable=noteVars[i], onvalue=1, offvalue=0, state="disabled") for i,n in enumerate(notes)]
    for n in noteButtons: n.pack(anchor=W)

    # Tuning key Frame
    keyVar.trace('w', validateKey)

    keyFrame = Frame(tuningFrame, borderwidth=3)
    keyFrame.pack(side='top', anchor=W)

    keyInputLabel = Label(keyFrame, text="(beta) Tune with key:")
    keyInputLabel.pack(side='left')
    keyInput = Entry(keyFrame,width=7,textvariable=keyVar)
    keyInput.pack(side='left')

    showKeyButton = Button(tuningFrame, text="Set Key", command=processTuningKey)
    showKeyButton.pack(side='top', anchor=W)

    root.mainloop()

if __name__ == "__main__": main()


from math import log

# Stores notes of the chromatic scale for referencing
notes=['A','A#','B','C','C#','D','D#','E','F','F#','G','G#']

# Constructs a major scale starting from a given number note
default_scales ={
    'Major Scale': [0,2,4,5,7,9,11],
    'Natural Minor Scale': [0,2,3,5,7,8,10],
    'Harmonic Minor Scale': [0,2,3,5,7,8,11],
    'Melodic Minor Scale': [0,2,3,5,7,9,11]
}

def construct_default_scale(note: int, scale_type: str) -> list:
    return [(note+n)%12 or 12 for n in default_scales[scale_type]]

# Converts notes in string form to their number equivalents, with A as key 1
def str_scale_to_numbers(scale: "list[str]") -> "list[int]":
    return sorted([notes.index(note.upper())+1 for note in scale])

# Converts notes in number form to their string equivalents, with A as key 1
def num_scale_to_strs(scale: "list[int]") -> "list[str]":
    return [notes[note-1] for note in sorted(scale)]

# Converts note number to frequency
def to_freq(note: int) -> float:
    return 27.5 * (2 ** ((note - 1) / 12))

# Converts frequency to note number
def to_note(freq: float) -> int:
    return round(log(freq/27.5, 2**(1/12))+1,4)

# Get closest element to a target item in a list
def get_closest(pool: list, target: float) -> float:
    return min(pool, key=lambda x: abs(x-target))

# Get all valid note frequencies given a scale of notes
def construct_note_freqs(scale: "list[int]") -> "list[float]":
    return [to_freq((12*i)+j) for i in range(8) for j in scale if 12*i+j<=88]


import setuptools

# with open("README.md", "r", encoding="utf-8") as fh:
#     long_description = fh.read()

setuptools.setup(
    name="spl_widgets",
    version="0.2.16",
    author="Colin Simon-Fellowes",
    author_email="colin.tsf@gmail.com",
    description="Widgets for the Barnard Speech Perception Laboratory",
    # long_description=long_description,
    # long_description_content_type="text/markdown",
    url="https://github.com/ctsf1/spl_widgets",
    project_urls={
        "Changelog":"https://github.com/ctsf1/spl_widgets#changelog"
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    install_requires=['pandas'],
    python_requires=">=3.6",
)
